#%%
from io import StringIO
import ruamel.yaml
yaml = ruamel.yaml.YAML()

book_config_file="_config.yml"

import jgtset as jset
jgtset_included_keys = "_jgtset_included.json"

updated_yaml_data=jset.update_jgt_on_existing_yaml_file(custom_path=jgtset_included_keys, target_filepath=book_config_file)
# %%
stream = StringIO()
yaml.dump(updated_yaml_data, stream)
print(stream.getvalue())






import os
import sys
import json
import ruamel.yaml;yaml:ruamel.yaml.YAML = ruamel.yaml.YAML()
from io import StringIO
import tempfile

import dotenv


sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))
from jgtcliconstants import JGTSET_CLI_DESCRIPTION, JGTSET_CLI_EPILOG, JGTSET_CLI_PROG_NAME,_JGTSET_EXCLUDED_ENV_EXPORT_KEYS
from jgtenv import get_dotenv_jgtset_export_path, is_dotjgt_env_sh_exists, load_dotjgt_env_sh
from jgterrorcodes import JGTFILES_EXIT_ERROR_CODE, JGTSETTING_EXIT_ERROR_CODE

import jgtcommon

def parse_args():

    parser = jgtcommon.new_parser(JGTSET_CLI_DESCRIPTION, JGTSET_CLI_EPILOG,prog=JGTSET_CLI_PROG_NAME, enable_specified_settings=True)
    #parser=jgtcommon.add_settings_argument(parser)
    parser.add_argument('-E','-env','--export-env', action='store_true', help='Export settings as environment variables')
    #env keys to export
    parser.add_argument('-K','-keys','--keys', nargs='+', help='Export only the specified keys as environment variables')
    
    parser.add_argument('-J','-json','--json', action='store_true', help='Print as JSON')
    
    parser.add_argument('-Y','--yml','--yaml', action='store_true', help='Print as YAML')
    
    parser.add_argument('-S','--silent', action='store_true', help='silent output')
    
    #view list
    parser.add_argument('-V','--view', action='store_true', help='View settings list')
    
    #add -O --output to specify the env file to export to instead of the default
    parser.add_argument('-O','--output', help='Specify the output env file to export to')
    
    #-U --update-config _config.yml jgt:

    parser.add_argument('-U', '--update', nargs='*', help='Update the specified yaml file with the jgt settings', metavar=('target_yaml_file', 'target_key'),required=False)
    
    #-R --reset-config _config.yml jgt:
    parser.add_argument('-R', '--reset-jgt-config', action='store_true', help='Reset the specified yaml file with the jgt settings', required=False)

    
    args = jgtcommon.parse_args(parser)
    return args


def _init_dotenv_jgt_export_file(env_file=None):
    
    try:
        jgt_batch_path = get_dotenv_jgtset_export_path() if not env_file else env_file
        with open(jgt_batch_path,"w") as f:
            #f.write("#!/bin/bash\n")
            f.write("# This file is generated by JGTSettingsCLI (jgtset)\n")
            f.write("# It should not be edited manually\n" if not env_file else f"# It normally should not be edited manually but it is a custom file, therefore you might if you know what you are doing\n")
            #f.write("# This file is used to fix the list export issue in the environment variables\n")
            #f.write("# Load this file first and then load the .env or other environment variable files to ovveride.  This fixes the list export issue\n")
            
    except:
        exit(JGTFILES_EXIT_ERROR_CODE)



def _add_value_to_jgt_export_file(key,value,quiet=True,env_file=None):
    try:
        enquote = False
        var_type = type(value).__name__
        #print("key is of type:",var_type)
        fixed_value = value 
        if var_type == "bool":
            fixed_value = str(value).lower()
        else:
            if var_type == "list" or var_type == 'CommentedSeq':
                fixed_value = __format_list_to_string(value)
                enquote = True
                
        jgt_batch_path = get_dotenv_jgtset_export_path() if not env_file else env_file
        with open(jgt_batch_path,"a") as f:
            if enquote:
                append_line = f"{key}=\"{fixed_value}\"\n"
            else:
                append_line = f"{key}={fixed_value}\n"
            if not quiet:print(f"export {key}={fixed_value}")
            f.write(append_line.replace('""','"'))
    except:
        exit(JGTFILES_EXIT_ERROR_CODE)        

def export_keys_to_environ(_settings=None,keys=None,quiet=True,env_file=None,custom_path=None):
    _init_dotenv_jgt_export_file(env_file=env_file)
    _what_to_export = _get_data_filtered_by_exportable_keys(_settings, keys,custom_path)
    for key, value in _what_to_export.items():
        if key not in _JGTSET_EXCLUDED_ENV_EXPORT_KEYS:
            if keys is None:
                if value is not None:
                    _export_key(key, value,quiet,env_file=env_file)
            else:
                if key in keys:
                    _export_key(key, value,quiet,env_file=env_file)
    #run the batch file to fix the list export issue
    #os.system(f"source {_get_jgt_batch_filepath()}")
    #load using dotenv
    # for key in os.environ : 
    #     if key in settings:
    #         print(f"export {key}={os.getenv(key)}")
    # print("-------------------")
    #dotenv.load_dotenv(dotenv_path=get_jgt_env_export_path())
    # test_passed=True
    # for key in settings.keys():
    #     if not key in os.environ:
    #         test_passed=False
   
    # for key in os.environ : 
    #     if key in settings:
    #         value = os.getenv(key)
    #         os.environ[key] =value
    #         print(f"export {key}={value}")
    #         os.system(f"export {key}={value}")
    #print("Test passed:" if test_passed else "Test failed:")
    #print(os.getenv("columns_to_remove"))

def _export_key(key, value,quiet=True,is_subkey=False,env_file=None):
    if isinstance(value, dict): 
        #export key of the value as a string coma separated
        subkey_str_val='"' if not is_subkey else ""
        c=0
        for subkey, subvalue in value.items():
            delimiter = "," if c < len(value)-1 else ""
            subkey_var_name = f"{key}_{subkey}"
            subkey_str_val+=subkey+delimiter
            if isinstance(subvalue, dict):                
                _export_key(subkey_var_name, subvalue,quiet,is_subkey=True,env_file=env_file)
            else:
                os.environ[subkey_var_name] = str(subvalue)
                _export_key(subkey_var_name, subvalue,quiet,is_subkey=True,env_file=env_file)
                #if not quiet:print(f"export {key}_{subkey}={subvalue}")
            c+=1
        #os.environ[key] = subkey_str_val
        subkey_str_val+='"' if not is_subkey else ""
        if  not is_subkey and subkey_str_val != "":
            #if not quiet:print(f"export {key}={subkey_str_val}")
            _add_value_to_jgt_export_file(key,subkey_str_val,quiet,env_file=env_file)
    else:
        if isinstance(value, list):
            #if list contains dict, dont export
            if len(value) > 0 and isinstance(value[0], dict):
                for i in range(len(value)):
                    for subkey, subvalue in value[i].items():
                        subkey_var_name = f"{key}_{i}_{subkey}"
                        os.environ[subkey_var_name] = str(subvalue)
                        _export_key(subkey_var_name, subvalue,quiet,is_subkey=True,env_file=env_file)
                        #if not quiet:print(f"export {key}_{i}_{subkey}={subvalue}")
                return
            list_fixed = __format_list_to_string(value)
            os.environ[key] = list_fixed
            _add_value_to_jgt_export_file(key,value,quiet,env_file=env_file)
            #if not quiet:print(f"export {key}={list_fixed}")
        else:
            os.environ[key] = str(value)
            _add_value_to_jgt_export_file(key,value,quiet,env_file=env_file)
            #if not quiet:print(f"export {key}={value}")

def __format_list_to_string(value,enquote=True,single_quote=False):
    list_fixed='"' if not single_quote else "'"
    if not enquote:
        list_fixed=""
    c=0
    for v in value:
        delimiter = ","
        if c == len(value)-1:
            delimiter = ""
        list_fixed+=str(v)+delimiter
        c+=1
    list_fixed+='"' if not single_quote and enquote  else "'" if single_quote and enquote else ""
    
    return list_fixed


def dump_as_json_output(_settings=None,keys=None,custom_path=None):
    _what_to_export = _get_data_filtered_by_exportable_keys(_settings, keys,custom_path)
    return json.dumps(_what_to_export, indent=2)

def dump_as_yaml_output(_settings=None,keys=None,custom_path=None):
    _what_to_export = _get_data_filtered_by_exportable_keys(_settings, keys,custom_path)
    #output to temp file then read it back, fuckin issue
    
    with tempfile.NamedTemporaryFile(delete=False, mode='w+') as temp_file:
        try:
            yaml.dump(_what_to_export, temp_file)
            temp_file.seek(0)
            stream_data = temp_file.read()
        except yaml.YAMLError as exc:
            print(exc)
    
    return stream_data

    # temp_file = tempfile.NamedTemporaryFile(delete=False)
    # temp_path = temp_file.name
    # with open(temp_path,'w') as stream:
    #     try:
    #         yaml.dump(_what_to_export,stream)
    #     except yaml.YAMLError as exc:
    #         print(exc)
    # with open(temp_path, 'r') as stream:
    #     try:
    #         stream_data = yaml.load(stream)
    #     except yaml.YAMLError as exc:
    #         print(exc)
    # print(type(stream_data))
    # return yaml.dump(stream_data)

def update_jgt_on_existing_yaml_file(target_filepath,_settings=None,keys=None,custom_path=None,target_key='jgt',add_on_only=True):
    jgtset_excluded=_JGTSET_EXCLUDED_ENV_EXPORT_KEYS
    if 'jgtset_excluded' in _settings:
        jgtset_excluded_=_settings['jgtset_excluded'].split(",")
        jgtset_excluded =jgtset_excluded+jgtset_excluded_
    
    jgt_yaml_content=dump_as_yaml_output(_settings,keys,custom_path)

    yaml_data_unfiltered=yaml.load(jgt_yaml_content)
    yaml_data=_get_data_filtered_by_exportable_keys(yaml_data_unfiltered, keys,custom_path)
    
    cfg={}
    with open(target_filepath, 'r') as stream:
        try:
            cfg = yaml.load(stream)
            #print(cfg)
        except yaml.YAMLError as exc:
            print(exc)
    
    
    if not add_on_only:
        cfg[target_key] = yaml_data
    else:
        if target_key in cfg:
            for key in yaml_data.keys():
                if key not in cfg[target_key]:
                    cfg[target_key][key] = yaml_data[key]
            #cfg[target_key].update(yaml_data)
        else:
            cfg[target_key] = yaml_data

    #clean up the excluded keys
    for key in jgtset_excluded:
        if key in cfg[target_key]:
            cfg[target_key].pop(key)
    
    os.rename(target_filepath, target_filepath+".bak")

    with open(target_filepath,'w') as temp_file:
        try:
            yaml.dump(cfg, temp_file)
            temp_file.seek(0)

        except yaml.YAMLError as exc:
            print(exc)

    with open(target_filepath, 'r') as stream:
        try:
            cfg = yaml.load(stream)
            #print(cfg)
        except yaml.YAMLError as exc:
            print(exc)
    return cfg
    
    

def _get_data_filtered_by_exportable_keys(data=None, keys=None,custom_path=None):
    if data is None:
        data = _load_settings(custom_path)
    _what_to_export = {}
    try:#jgtset_included
        if 'jgtset_included' in data and keys is None:
            keys=data['jgtset_included'].split(",")
    except:
        pass    
    try:#jgtset_included
        if 'JGTSET_INCLUDED' in os.environ and keys is None:
            keys=os.environ['JGTSET_INCLUDED'].split(",")
    except:
        pass
    if keys is not None:
        for key in keys:
            if key in data:
                _what_to_export[key] = data[key]
        for key in keys:
            if key not in _what_to_export and key in os.environ:
                _what_to_export[key] = os.environ[key]
        for key in keys:
            if key not in _what_to_export and is_dotjgt_env_sh_exists():
                load_dotjgt_env_sh()
                if key in os.environ:
                    _what_to_export[key] = os.environ[key]
    else:
        _what_to_export =data
    #remove keys that should not be exported using _JGTSET_EXCLUDED_ENV_EXPORT_KEYS
    for key in _JGTSET_EXCLUDED_ENV_EXPORT_KEYS:
        if key in _what_to_export:
            _what_to_export.pop(key)
    
    try:
        if os.environ['JGTSET_EXCLUDED']:
            for key in os.environ['JGTSET_EXCLUDED_ENV_EXPORT_KEYS'].split(","):
                if key in _what_to_export:
                    _what_to_export.pop(key)
    except:
        pass
    
    try:
        if 'jgtset_excluded' in data:
            for key in data['jgtset_excluded'].split(","):
                if key in _what_to_export:
                    _what_to_export.pop(key)
            _what_to_export.pop('jgtset_excluded')
    except:
        pass      
    return _what_to_export




def main():
    args = parse_args()
    env_flag = args.export_env
    json_flag = args.json
    yaml_flag = args.yml
    silent_flag = args.silent
    keys = args.keys
    output_file = args.output
    view_flag = args.view
    reset_jgt_flag = True if args.reset_jgt_config else False
    TARGET_YAML_FILE_DEFAULT = '_config.yml'
    TARGET_KEY_DEFAULT = 'jgt'
    target_yaml_file, target_key =TARGET_YAML_FILE_DEFAULT, TARGET_KEY_DEFAULT
    if hasattr(args,'update') and args.update is not None:
        if args.update == f'{TARGET_YAML_FILE_DEFAULT},{TARGET_KEY_DEFAULT}':
            target_yaml_file, target_key = TARGET_YAML_FILE_DEFAULT, TARGET_KEY_DEFAULT
        else:
            update_length = args.update.__len__()
            if update_length > 1:
                target_yaml_file, target_key = args.update.split(',')
            else:
                if update_length==1:
                    target_yaml_file = args.update[0]
                else:
                    target_yaml_file=TARGET_YAML_FILE_DEFAULT
                target_key = TARGET_KEY_DEFAULT
    else:
        target_yaml_file, target_key = None, None
    
    # if args.update:
    #     if len(args.update) == 0:
    #         target_yaml_file, target_key = '_config.yml', 'jgt'
    #     else:
    #         target_yaml_file, target_key = args.update
    try:
        
        print_output=   process_env(env_flag,keys=keys,json_flag=json_flag,yaml_flag=yaml_flag,silent_flag=silent_flag,output_file=output_file,view_flag=view_flag,target_yaml_file=target_yaml_file,target_key=target_key,reset_jgt_flag=reset_jgt_flag)
        
        print(print_output)
        
    except:
        print("Error loading settings")
        sys.exit(JGTSETTING_EXIT_ERROR_CODE)

def process_env(env_flag,keys=None,json_flag=False,yaml_flag=False,silent_flag=False,output_file=None,view_flag=False,custom_path=None,target_yaml_file=None,target_key=None,reset_jgt_flag=False):
    
    settings = _load_settings(custom_path)
    
    if view_flag:
        for key in settings.keys():
            print(key)
        return ""
    
    if env_flag and not json_flag and not yaml_flag and not target_yaml_file:
        export_keys_to_environ(settings,keys=keys,quiet=False if not silent_flag else True,env_file=output_file)
        return ""

    if target_yaml_file:
        updated_yaml_data = update_jgt_on_existing_yaml_file(target_yaml_file,settings,keys,target_key,add_on_only=not reset_jgt_flag)
        stream = StringIO()
        yaml.dump(updated_yaml_data, stream)
        print_output=stream.getvalue()
        return print_output
        

    print_output=""
    if yaml_flag:
        print_output = dump_as_yaml_output(settings,keys )
    else:
        print_output = dump_as_json_output(settings,keys )
    
    if env_flag: #Quiet export with other outputs
        export_keys_to_environ(settings,keys=keys,quiet=True,env_file=output_file)

    
    return print_output

def _load_settings(custom_path):
    return jgtcommon.get_settings(custom_path=custom_path)

if __name__ == '__main__':
    main()



import unittest
from unittest.mock import patch
from jgtfxhelper import mkfn_cfxdata_filepath

class TestMkfnCfxdataFilepath(unittest.TestCase):
    @patch('jgtfxhelper.mkfn_cdata_filepath')
    def test_mkfn_cfxdata_filepath_use_local(self, mock_mkfn_cdata_filepath):
        mock_mkfn_cdata_filepath.return_value = '/data/jgt/filename.txt'
        result = mkfn_cfxdata_filepath('filename.txt.txt', use_local=True,ext='txt')
        self.assertEqual(result, '/data/jgt/filename.txt')


    @patch('jgtfxhelper.get_data_path')
    def test_mkfn_cfxdata_filepath_not_use_local(self, mock_get_data_path):
        mock_get_data_path.return_value = '/data'
        result = mkfn_cfxdata_filepath('filename.txt', use_local=False)
        self.assertEqual(result, '/data/filename.txt')

if __name__ == '__main__':
    unittest.main()


import datetime
import time
import os

import sys
import subprocess

from jgtutils.jgterrorcodes import SUBPROCESS_RUN_ERROR_EXIT_ERROR_CODE

sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

import jgtcommon
from jgtclihelper import print_jsonl_message

import jgtwslhelper as wsl


def refreshTF( timeframe:str,quote_count:int=-1, quiet:bool=True,use_full:bool=False,verbose_level=0,tlid_range=None,keep_bid_ask=False):
  #if not quiet:
  print(f"Refreshing  {timeframe}")
  try:
    print("WE Would be processing our stuff: " + str(datetime.datetime.now()))
    #wsl.getPH(instrument, timeframe,quote_count=quote_count, tlid_range=tlid_range, use_full=use_full,verbose_level=verbose_level,keep_bid_ask=keep_bid_ask)
    return timeframe
  except Exception as e:
    print("Error in refreshPH")
    raise e

timeframe=None
current_time=None
EXIT_ON_ERROR=False

from jgtclihelper import build_jsonl_message
def _exit_quietly_handler():
  global timeframe
  msg = f"Wait canceled by user"
  o=build_jsonl_message(msg,extra_dict={"timeframe":timeframe},state="canceled",scope=APP_SCOPE,use_short=True)
  return o

def parse_args():
    parser = jgtcommon.new_parser("JGT Cron Test","Test launching or unlocking (exit when specific timeframes arrives.)",enable_specified_settings=True,exiting_quietly_handler=_exit_quietly_handler)#add_exiting_quietly_flag=True,exiting_quietly_message=f"")
    #parser=jgtcommon.add_settings_argument(parser)
    #parser=jgtcommon._preload_settings_from_args(parser)
    
    parser= jgtcommon.add_timeframe_standalone_argument(parser,required=True,from_jgt_env=True)
    
    step_group=parser.add_mutually_exclusive_group()
    #exit the program when the timeframe is reached
    step_group.add_argument("-X", "--exit", action="store_true", help="Exit the program when the timeframe is reached.")
    
    #--script-to-run
    step_group.add_argument("-S", "--script-to-run", help="Script to run when the timeframe is reached. (.jgt/tfw.sh)")
    #--cli-to-run
    step_group.add_argument("-C", "--cli-to-run", help="CLI to run when the timeframe is reached. (python -m jgtutils.cli_test_cronrun_helper)",nargs="*")
    #--function
    step_group.add_argument("-F", "--function", help="Function to run when the timeframe is reached.")
    
    #--message
    parser.add_argument("-M", "--message", help="Message to display when the timeframe is reached.",default="Timeframe reached.")
    
    parser.add_argument("-I", "--in-message", help="Message to display when the timeframe wait starts.",default="Timeframe waiting started")
    
    
    #--nooutput
    parser.add_argument("-N", "--no-output", action="store_true", help="Do not output anything.")
    #parser=jgtcommon.add_bars_amount_V2_arguments(parser)
    #parser=jgtcommon.add_keepbidask_argument(parser)
    #parser=jgtcommon.add_tlid_range_argument(parser)
    parser=jgtcommon.add_verbose_argument(parser)
    
    args = jgtcommon.parse_args(parser)
    return args
APP_SCOPE="tfwait"
def main():
    global timeframe,current_time
    #add_exiting_quietly()
    
    args = parse_args()
    timeframe = args.timeframe
    
    ctx_times = get_times_by_timeframe_str(timeframe)
    quiet = args.quiet
    if not quiet:
      print(f"CTX times: {ctx_times}")
    sleep_duration = 60 if timeframe != "m1" else 2
    

    if not args.no_output:
      _print_app_message(timeframe,args.in_message,state="started" )
    while True:
      current_time = get_current_time(timeframe)
     
      
      if current_time in ctx_times:  # Adjust the times as needed
          adjusted_sleep_duration = sleep_duration
          if args.exit:
              #print(args.message," ",args.timeframe," reached at ",current_time)
              if not args.no_output:
                _print_app_message(timeframe, args.message,state="reached")
              #sys.stdout.flush()
              exit(0)
          if args.script_to_run:
            #@STCIssue We would need to calculate the time to run the script and substract that to the sleep_duration
            try:
              before_run = datetime.datetime.now()
              _run_script_to_run(args.script_to_run)
              after_run = datetime.datetime.now()
              elapsed = after_run - before_run
              elapsed_seconds = elapsed.total_seconds()
              adjusted_sleep_duration = sleep_duration - elapsed_seconds
              if adjusted_sleep_duration < 0:
                adjusted_sleep_duration = 1
            except Exception as e:
              msg = f"Error running script{args.script_to_run}: {e}"
              print_jsonl_message(msg,extra_dict={"timeframe":timeframe,"time":get_current_time(timeframe)},scope=APP_SCOPE,state="error",use_short=True)
              if EXIT_ON_ERROR:
                exit(SUBPROCESS_RUN_ERROR_EXIT_ERROR_CODE)
          elif args.cli_to_run:
            try:
              subprocess.run(args.cli_to_run, check=True)
            except Exception as e:
              msg = f"Error running app {args.cli_to_run}: {e}"
              print_jsonl_message(msg,extra_dict={"timeframe":timeframe,"time":get_current_time(timeframe)},scope=APP_SCOPE,state="error",use_short=True)
              if EXIT_ON_ERROR:
                exit(SUBPROCESS_RUN_ERROR_EXIT_ERROR_CODE)
          else:
            output = refreshTF(args.timeframe)
            print(output)
          time.sleep(adjusted_sleep_duration)  # Sleep for x seconds to avoid multiple runs within the same minute
      time.sleep(1)  # Check every second
      #print(".", end="")

def _run_script_to_run(script_to_run):
  global timeframe,current_time  
  if os.path.exists(script_to_run):
      subprocess.run(["bash",script_to_run,timeframe,current_time], check=True)
  else:
      print(f"Script {script_to_run} not found.")

def _run_function(function_to_run):
  global timeframe,current_time  
  if os.path.exists("/opt/binscripts/load.sh"):
      subprocess.run(["bash","-c",". ",timeframe,current_time], check=True)
  else:
      print(f"Script {script_to_run} not found.")

def _print_app_message(timeframe, msg,state=None,use_short=True):
    print_jsonl_message(msg,extra_dict={"timeframe":timeframe,"time":get_current_time(timeframe)},scope=APP_SCOPE,state=state,use_short=use_short)

def get_current_time(timeframe):
    return datetime.datetime.now().strftime("%H:%M") if timeframe != "m1" else datetime.datetime.now().strftime("%H:%M:%S")

def get_times_by_timeframe_str(timeframe:str):
  if timeframe=="D1" or timeframe=="W1" or timeframe=="M1":
    return get_timeframe_daily_ending_time()
  if timeframe=="H8":
    return get_timeframes_times_by_minutes(8*60)
  if timeframe=="H4":
    return get_timeframes_times_by_minutes(4*60)
  if timeframe=="H3":
    return get_timeframes_times_by_minutes(3*60)
  if timeframe=="H2":
    return get_timeframes_times_by_minutes(2*60)
  if timeframe=="H1":
    return get_timeframes_times_by_minutes(60)
  if timeframe=="m30":
    return get_timeframes_times_by_minutes(30)
  if timeframe=="m15":
    return get_timeframes_times_by_minutes(15)
  if timeframe=="m5":
    return get_timeframes_times_by_minutes(5)
  if timeframe=="m1":
    return get_timeframes_times_by_minutes(1)

def get_timeframes_times_by_minutes(minutes:int):
    start_range = 0
    if minutes>=60:
        start_range = 1 # we start at 1:00 for those timeframes
    if minutes>1:
      return [f"{str(h).zfill(2)}:{str(m).zfill(2)}" for h in range(start_range,24) for m in range(0,60,minutes)]
    elif minutes==1:
      return [f"{str(h).zfill(2)}:{str(m).zfill(2)}:00" for h in range(start_range,24) for m in range(0,60) ] +  [f"{str(h).zfill(2)}:{str(m).zfill(2)}:01" for h in range(start_range,24) for m in range(0,60) ] # We are sure to process m1 at the end of the minute if by any chance we are late of 1 second

def get_timeframe_daily_ending_time() -> str:
    #if timeframe != "D1":
    #    raise ValueError("This function only handles the D1 timeframe.")
    
    now = datetime.datetime.now()
    year = now.year
    
    # Assuming DST starts on the second Sunday in March and ends on the first Sunday in November
    dst_start = datetime.datetime(year, 3, 8) + datetime.timedelta(days=(6 - datetime.datetime(year, 3, 8).weekday()))
    dst_end = datetime.datetime(year, 11, 1) + datetime.timedelta(days=(6 - datetime.datetime(year, 11, 1).weekday()))
    
    if dst_start <= now < dst_end:
        return "22:00:00"
    else:
        return "21:00:00"



if __name__ == '__main__':
    main()
    



import unittest
from unittest.mock import patch, MagicMock
from FXTransact import FXTrade

class TestFXTrade(unittest.TestCase):
    @patch('FXTransact.FXTrade.from_id')
    def test_from_id_valid_trade_id(self, mock_from_id):
        trade_id = 123
        mock_trade = MagicMock(spec=FXTrade)
        mock_trade.trade_id = trade_id
        mock_from_id.return_value = mock_trade

        trade = FXTrade.from_id(trade_id)
        self.assertIsInstance(trade, FXTrade)
        self.assertEqual(trade.trade_id, trade_id)

    @patch('FXTransact.FXTrade.from_id')
    def test_from_id_invalid_trade_id(self, mock_from_id):
        trade_id = -1
        mock_from_id.return_value = None

        trade = FXTrade.from_id(trade_id)
        self.assertIsNone(trade)
        
    

if __name__ == '__main__':
    unittest.main()


DOTJGTENV_NOT_FOUND_EXIT_ERROR_CODE = 727

SUBPROCESS_RUN_ERROR_EXIT_ERROR_CODE = 729
INSTRUMENT_NOT_VALID_EXIT_ERROR_CODE = 600
ORDER_ADDING_FAILED_EXIT_ERROR_CODE = 5
ORDER_NOT_FOUND_EXIT_ERROR_CODE = 3
ORDER_ARGUMENTS_INVALID_EXIT_ERROR_CODE = 10

TRADE_CLOSE_ARGUMENTS_INVALID_EXIT_ERROR_CODE = 107
TRADE_AMOUNT_TO_CLOSE_INVALID_EXIT_ERROR_CODE = 108
TRADE_NOT_CLOSED_EXIT_ERROR_CODE = 110
TRADE_NOT_OPENED_EXIT_ERROR_CODE = 115
TRADE_NO_OPEN_POSITION_EXIT_ERROR_CODE = 120
TRADE_ALREADY_OPENED_EXIT_ERROR_CODE = 133
TRADE_ALREADY_CLOSED_EXIT_ERROR_CODE = 135

TRADE_ALREADY_EXISTS_EXIT_ERROR_CODE = 140

JGTSETTING_EXIT_ERROR_CODE = 666

JGTFILES_EXIT_ERROR_CODE = 700

#SEMANTICS
TRADE_NOT_FOUND_EXIT_ERROR_CODE = 105
TRADE_NOT_EXISTS_EXIT_ERROR_CODE = 105

TRADE_NOT_MODIFIED_EXIT_ERROR_CODE =    150
TRADE_STOP_CHANGING_EXIT_ERROR_CODE =   155
TRADE_STOP_NOT_CHANGED_EXIT_ERROR_CODE = 160
TRADE_STOP_INVALID_EXIT_ERROR_CODE = 165

SUBSCRIPTION_MANAGEMENT_EXIT_ERROR_CODE = 544

MARKET_CLOSED_EXIT_ERROR_CODE = 305

ACCOUNT_NOT_FOUND_EXIT_ERROR_CODE = 201
ACCOUNT_EXIT_ERROR_CODE = 202

ORDER_WITH_SAME_ENTRY_DATA_EXISTS_EXIT_ERROR_CODE = 176
ORDER_REMOVED_EXIT_ERROR_CODE = 80
ORDER_STOP_INVALID_EXIT_ERROR_CODE = 165

STOP_INVALID_EXIT_ERROR_CODE = 165

ORDER_REMOVAL_FAILED_BECOMED_A_TRADE_EXIT_ERROR_CODE = 18
FX_RESPONSE_TIMEOUT_EXIT_ERROR_CODE = 22

MARKET_DATA_UPDATE_FAILED_EXIT_ERROR_CODE = 322

ORDER_PRICE_EXCEEDS_LIMIT_EXIT_ERROR_CODE=99 #Indicates that the order price exceeds the configured price limit.
INVALID_ORDER_QUANTITY_EXIT_ERROR_CODE=95 #Indicates that the order quantity is invalid (e.g., too small, too large, or not a multiple of the minimum lot size).

"""

# Analysis and Suggestions

The provided Python constants for error codes seem comprehensive and well-defined. Here are some suggestions for improvement:

Consistency: Consider using a consistent naming convention for the constants. For example, all constants could be in uppercase with underscores separating words.
Documentation: Add comments to each constant explaining the specific error condition it represents. This will improve code readability and maintainability.
Completeness: Review the application's code to ensure all potential error conditions are covered by the defined constants.
Error Handling: Implement proper error handling mechanisms in the application to catch and handle these errors gracefully. This could involve logging the errors, notifying the user, or taking corrective actions.
Testing: Write unit tests to ensure that the error codes are used correctly and that the error handling logic functions as expected.

# Additional Error Codes
Based on common trading scenarios, here are some additional error codes you might consider adding:

INSUFFICIENT_FUNDS_EXIT_ERROR_CODE: Indicates that the account does not have enough funds to execute the requested order.
INVALID_ORDER_TYPE_EXIT_ERROR_CODE: Indicates that the order type specified is not supported by the exchange or the application.
INVALID_ORDER_QUANTITY_EXIT_ERROR_CODE: Indicates that the order quantity is invalid (e.g., too small, too large, or not a multiple of the minimum lot size).
ORDER_PRICE_EXCEEDS_LIMIT_EXIT_ERROR_CODE: Indicates that the order price exceeds the configured price limit.
CONNECTION_ERROR_EXIT_ERROR_CODE: Indicates that the application could not connect to the exchange or data provider.
API_ERROR_EXIT_ERROR_CODE: Indicates that an error occurred while communicating with the exchange's API.
TIMEOUT_ERROR_EXIT_ERROR_CODE: Indicates that a request to the exchange timed out.

"""



import unittest
from jgtos import fix_path_ext

class TestFixPathExt(unittest.TestCase):
    def test_fix_path_ext(self):
        # Test case 1: Existing extension is replaced with the desired extension
        fpath = "path/to/file.csv.csv"
        ext = "csv"
        expected_result = "path/to/file.csv"
        result = fix_path_ext(ext, fpath)
        self.assertEqual(result, expected_result)

        # Test case 2: Existing extension is replaced with the desired extension (with multiple dots in the file name)
        fpath = "path/to/file.name.csv"
        ext = "csv"
        expected_result = "path/to/file.name.csv"
        result = fix_path_ext(ext, fpath)
        self.assertEqual(result, expected_result)

        # Test case 3: Existing extension is replaced with the desired extension (with no existing extension)
        fpath = "path/to/file"
        ext = "csv"
        expected_result = "path/to/file.csv"
        result = fix_path_ext(ext, fpath)
        self.assertEqual(result, expected_result)

        # Test case 4: Existing extension is replaced with the desired extension (with no existing extension and multiple dots in the file name)
        fpath = "path/to/file.name"
        ext = "csv"
        expected_result = "path/to/file.name.csv"
        result = fix_path_ext(ext, fpath)
        self.assertEqual(result, expected_result)

        # Test case 5: Existing extension is replaced with the desired extension (with existing extension different from the desired extension)
        fpath = "path/to/file.txt"
        ext = "txt"
        expected_result = "path/to/file.txt"
        result = fix_path_ext(ext, fpath)
        self.assertEqual(result, expected_result)

        # # Test case 6: Existing extension is replaced with the desired extension (with existing extension same as the desired extension)
        fpath = "path/to/file"
        ext = None
        exception_message=f"No extension found in the file path: {fpath}"
        with self.assertRaises(Exception) as context:
            result = fix_path_ext(ext, fpath)
        self.assertEqual(exception_message, str(context.exception))
          
          

if __name__ == '__main__':
    unittest.main()


"""
jgtutils package
"""

version='0.2.177'

import sys
import os

sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

from jgtos import (tlid_range_to_jgtfxcon_start_end_str,
                   tlid_range_to_start_end_datetime)

import jgtcommon as common
import jgtos as jos
import jgtpov as pov
import jgtwslhelper as wsl
from jgtcommon import readconfig,new_parser,parse_args,load_settings,get_settings
from jgtpov import calculate_tlid_range as get_tlid_range
from FXTransact import (FXTransactDataHelper as ftdh,
                        FXTransactWrapper as ftw)

from jgtclihelper import (print_jsonl_message as printl)

from jgtenv import load_dotjgt_env_sh,load_dotjgtset_exported_env,load_dotfxtrade_env,load_env

# def load_env():
#   _load_dotjgt_env_sh=load_dotjgt_env_sh()
#   _load_dotjgtset_exported_env=load_dotjgtset_exported_env()
#   _load_dotfxtrade_env=load_dotfxtrade_env()
#   if _load_dotjgt_env_sh and _load_dotjgtset_exported_env and _load_dotfxtrade_env:
#     return True
#   return False
    

def load_logging():
  from jgtutils import jgtlogging as jlog





import os

OFFERS_CSV_DATA="""18,CAD/JPY
1010,SPX500
2020,WHEATF
3008,2USNote
3,GBP/USD
36,EUR/NOK
14,EUR/AUD
3004,5USNote
17,AUD/JPY
32,EUR/SEK
21,GBP/NZD
2003,SOYF
1035,EUSTX50
1002,ESP35
89,NZD/CHF
96,EUR/HUF
1013,US30
3011,FED30D
4,USD/CHF
22,GBP/AUD
13,GBP/CHF
1001,AUS200
15,EUR/CAD
90,CAD/CHF
1012,UK100
4001,XAU/USD
3005,10USNote
39,AUD/CHF
91,NZD/CAD
2,USD/JPY
30,USD/SEK
5,EUR/CHF
38,USD/MXN
83,USD/TRY
3010,Schatz
1008,NAS100
97,USD/HUF
1,EUR/USD
20,GBP/CAD
16,AUD/CAD
1060,US2000
1004,GER30
12,CHF/JPY
50,USD/HKD
37,USD/NOK
2001,USOil
1058,USDOLLAR
1007,JPN225
98,TRY/JPY
71,ZAR/JPY
9,EUR/GBP
47,USD/ZAR
105,USD/CNH
4002,XAG/USD
2021,CORNF
6,AUD/USD
11,GBP/JPY
1003,FRA40
3001,Bund
2015,NGAS
2002,UKOil
8,NZD/USD
1005,HKG33
3012,EURIBOR3M
19,NZD/JPY
40,EUR/NZD
28,AUD/NZD
3014,SONIA3M
3009,Bobl
1016,Copper
87,EUR/TRY
7,USD/CAD
10,EUR/JPY"""

def offer_id_to_instrument(offer_id):
    for line in OFFERS_CSV_DATA.split("\n"):
        if line.startswith(str(offer_id) + ","):
            return line.split(",")[1]
    return None

def instrument_to_offer_id(symbol):
    for line in OFFERS_CSV_DATA.split("\n"):
        if line.split(",")[1] == symbol:
            return int(line.split(",")[0])
    return None

def offers_to_dict():
    dict = {}
    for line in OFFERS_CSV_DATA.split("\n"):
        dict[int(line.split(",")[0])] = line.split(",")[1]
    return dict

def instruments_to_dict():
    dict={}
    for line in OFFERS_CSV_DATA.split("\n"):
        dict[line.split(",")[1]] = int(line.split(",")[0])
    return dict


from jgtos import ensure_directory_exists, fix_path_ext, mkfn_cdata_filepath
from jgtcliconstants import JGT_FXDATA_NS
from jgtos import get_data_path
def mkfn_cfxdata_filepath(fn,use_local=True,ext=None):
    #.replace(f".{ext}.{ext}",f".{ext}")
    if use_local:
        fpath = mkfn_cdata_filepath( fn)

    else :
        fx_dir_path=get_data_path(JGT_FXDATA_NS)
        fpath = os.path.join(fx_dir_path,fn)
    if ext is not None:
        fpath = fix_path_ext(ext, fpath)
    cleaned_filepath = fpath.replace("_.",".")
    #ensure_directory_exists(cleaned_filepath)
    return cleaned_filepath



def is_entry_stop_valid(entry_rate,stop_rate,bs):
    """
    Entry Rate and Stop Rate validation
    
    """
    if bs=="S":
        if entry_rate<stop_rate:
            return True
        else:
            return False
    elif bs=="B":
        if entry_rate>stop_rate:
            return True
        else:
            return False


import os

import sys

sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

import jgtcommon

def parse_args():
    parser = jgtcommon.new_parser("Timeframe Test settings","Test loading custom timeframes from settings",enable_specified_settings=True)
    #parser=jgtcommon.add_settings_argument(parser)
    #parser=jgtcommon._preload_settings_from_args(parser)
    
    parser= jgtcommon.add_instrument_standalone_argument(parser)
    parser= jgtcommon.add_timeframe_standalone_argument(parser)
    
    args = jgtcommon.parse_args(parser)
    return args

from jgtconstants import INSTRUMENTS_DEFAULT, TIMEFRAMES_DEFAULT
def main():
    args = parse_args()
    instruments=jgtcommon.get_instruments(INSTRUMENTS_DEFAULT)
    timeframes=jgtcommon.get_timeframes(TIMEFRAMES_DEFAULT)
    print("instruments:",instruments)
    print("timeframes:",timeframes)
    timeframes2=jgtcommon.get_timeframes(None,"_default_timeframes_a")
    print("_default_timeframes_a:",timeframes2)
    instruments2=jgtcommon.get_instruments(None,"instruments")
    print("instruments2:",instruments2)
    
    
if __name__ == '__main__':
    main()
    



import json
import sys
def print_jsonl_message(msg,extra_dict:dict=None,scope=None,state:str=None,msg_key_name = "message",state_key_name = "state",scope_key = "scope",use_short=False):
    json_output = build_jsonl_message(msg,extra_dict,scope,state,msg_key_name,state_key_name,scope_key,use_short)
    print(json_output)

def build_jsonl_message(msg,extra_dict:dict=None,scope=None,state:str=None,msg_key_name = "message",state_key_name = "state",scope_key = "scope",use_short=False):
    o={}
    if use_short:
        msg_key_name="msg"
        state_key_name="s"
        scope_key="sc"
    o[msg_key_name]=msg
    if extra_dict:
        o.update(extra_dict)
    if scope:
        o[scope_key]=scope
    if state:        
        o[state_key_name]=state
    json_output = json.dumps(o)
    return json_output


_exit_message = None
_exit_handler = None

def signal_handler(sig, frame):
    global _exit_message, _exit_handler
    if _exit_message:
        print(_exit_message)
    if _exit_handler:
        #call it        
        msg=_exit_handler()
        print(msg)
    sys.exit(0)

def add_exiting_quietly(message:str=None,exit_handler=None):
    global _exit_message, _exit_handler
    if exit_handler:
        _exit_handler = exit_handler
    if message:
        _exit_message = message
    import signal
    signal.signal(signal.SIGINT, signal_handler)


import unittest
from unittest.mock import patch
from jgtfxhelper import mkfn_cfxdata_filepath
from FXTransact import FXTrade

class TestFXTrade(unittest.TestCase):
    def test_tofn_default_filename(self):
        trade = FXTrade(
            trade_id=123,
            account_id=456,
            account_name='Test Account',
            account_kind=1,
            offer_id=789,
            amount=1000,
            buy_sell='buy',
            open_rate=1.2345,
            open_time='2022-01-01 10:00:00',
            open_quote_id='quote123',
            open_order_id='order123',
            open_order_req_id='req123',
            open_order_request_txt='Test Order',
            commission=0.0,
            rollover_interest=0.0,
            trade_id_origin='',
            used_margin=0.0,
            value_date='',
            parties='',
            dividends=0.0,
            pl=0.0,
            gross_pl=0.0,
            close=0.0,
            stop=0.0,
            limit=0.0,
            stop_order_id='',
            limit_order_id='',
            instrument='EUR/USD',
            trail_rate=0.0,
            trail_step=0.0,
            close_commission=0.0,
            message=''
        )
        result = trade.tofn()
        self.assertEqual(result, 'trade_123.json')

    def test_tofn_custom_filename(self):
        trade = FXTrade(
            trade_id=123,
            account_id=456,
            account_name='Test Account',
            account_kind=1,
            offer_id=789,
            amount=1000,
            buy_sell='buy',
            open_rate=1.2345,
            open_time='2022-01-01 10:00:00',
            open_quote_id='quote123',
            open_order_id='order123',
            open_order_req_id='req123',
            open_order_request_txt='Test Order',
            commission=0.0,
            rollover_interest=0.0,
            trade_id_origin='',
            used_margin=0.0,
            value_date='',
            parties='',
            dividends=0.0,
            pl=0.0,
            gross_pl=0.0,
            close=0.0,
            stop=0.0,
            limit=0.0,
            stop_order_id='',
            limit_order_id='',
            instrument='EUR/USD',
            trail_rate=0.0,
            trail_step=0.0,
            close_commission=0.0,
            message=''
        )
        result = trade.tofn(ext='yaml', fn='my_trade')
        self.assertEqual(result, 'my_trade123.yaml')

if __name__ == '__main__':
    unittest.main()


import os
import sys

from dotenv import load_dotenv

from jgtcliconstants import JGT_ENV_EXPORT_NAME, JGT_SUBDIR_NAME,JGT_FXTRADE_ENV_FILENAME

def get_dotfxtrade_env_path():
    return os.path.join(os.getcwd(),JGT_FXTRADE_ENV_FILENAME)

def load_dotfxtrade_env():
    dotfxtrade_env_path = get_dotfxtrade_env_path()
    if os.path.exists(dotfxtrade_env_path):
        load_dotenv(dotenv_path=dotfxtrade_env_path)
        return True
    else:
        return False

def is_dotfxtrade_env_exists():
    return os.path.exists(get_dotfxtrade_env_path())

def get_dotjgt_env_sh_path():
    return os.path.join(os.getcwd(),".jgt","env.sh")

def load_dotjgt_env_sh():
    dotjgt_env_sh_path = get_dotjgt_env_sh_path()
    if os.path.exists(dotjgt_env_sh_path):
        load_dotenv(dotenv_path=dotjgt_env_sh_path)
        return True
    else:
        return False

def is_dotjgt_env_sh_exists():
    return os.path.exists(get_dotjgt_env_sh_path())

def load_dotjgtset_exported_env():
  dotenv_path=get_dotenv_jgtset_export_path()
  if os.path.exists(dotenv_path):
    load_dotenv(dotenv_path=dotenv_path)
    return True
  else:
    return False

def load_env():
  _load_dotjgt_env_sh=load_dotjgt_env_sh()
  _load_dotjgtset_exported_env=load_dotjgtset_exported_env()
  _load_dotfxtrade_env=load_dotfxtrade_env()
  _load_jgtyaml_env=load_jgtyaml_env()
  if _load_dotjgt_env_sh or _load_dotjgtset_exported_env or _load_dotfxtrade_env or _load_jgtyaml_env:
    return True
  return False

import yaml
def load_jgtyaml_env(config_file="_config.yaml",jgt_key="jgt"):
  try:
    with open(config_file) as f:
      config = yaml.safe_load(f)
      jgt_env=config[jgt_key]
      for key in jgt_env:
        os.environ[key]=jgt_env[key]
      return True
  except Exception as e:
    return False
  

def get_dotenv_jgtset_export_path(in_jgt_subdir=False):
    
    if in_jgt_subdir:
      jgt_export_directory = os.path.join(os.getcwd(),JGT_SUBDIR_NAME)
      os.makedirs(jgt_export_directory, exist_ok=True)#|print(f"Directory {subdir} created")
    else :
      jgt_export_directory = os.getcwd()
      
    batch_file_path = os.path.join(jgt_export_directory,JGT_ENV_EXPORT_NAME)
    return batch_file_path
  
def get_openai_key():
    """Reads the OpenAI API key from the environment or a .env file."""

    # Define the possible locations for the .env file
    dotenv_paths = [
      os.path.join(os.path.dirname(__file__), '..', '.env'),  # Parent directory
      os.path.join(os.path.dirname(__file__), '..', '..', '.env'),  # Grandparent directory
      os.path.join(os.path.expanduser("~"), ".env"),  # Home directory
    ]

    # Try to load the .env file from the possible locations
    for dotenv_path in dotenv_paths:
      if os.path.exists(dotenv_path):
        load_dotenv(dotenv_path)
        break

    # Get the API key from the environment variables
    api_key = os.getenv("OPENAI_API_KEY")

    # Raise an error if the API key is not found
    if api_key is None:
      raise ValueError(
        "OPENAI_API_KEY not found in environment variables or .env file."
      )
    return api_key
